% Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification"
% This code reproduces Figures 2(a)-(b) and 4, and Tables S1 and S2 in the appendix
% (up to some randomness due to different seeds)

clear
clc
close all

N=500;

% support of X
NX=4;
pi_X=(1/NX:1/NX:(NX-1)/NX);

X=zeros(N,1);

UX=rand(N,1);

for kX=1:NX
    if kX==1
        indX=(UX<pi_X(1));
    elseif kX>1 && kX<NX
        indX=(UX>=pi_X(kX-1)).*(UX<pi_X(kX));
    elseif kX==NX
        indX=(UX>=pi_X(NX-1));
    end
    
    X(indX==1)=(kX-1/2)/NX;
end

Xtot=[X ones(N,1)];

[~,dimbeta]=size(Xtot);

beta=[2;-1];

% plot density of X (Figure 2a)
figure
histogram(kron([1/8:1/4:1],ones(1,25)),8,'binwidth',.01)
hold on
histogram(kron([1/40:1/20:1],ones(1,4)),40,'binwidth',.01)
axis([-0.5 1.5 0 30])
xlabel('X value','FontSize',15)
ylabel('frequency (percent)','FontSize',15)
hold off

% discretization
KK=500;

a_kk=sort(randn(KK,1),1);

Mat=diag(normcdf(a_kk(2:end))-normcdf(a_kk(1:end-1)));
D2=Mat;

invD2=inv(D2);

% number of simulations
Nsim=1000;

par0=zeros(dimbeta,1);

options=optimoptions('fminunc','Display','off');

lambdagrid=(1:1:3)';
[nlambdagrid,~]=size(lambdagrid);

epsgrid=(.0001:.05:3)';
[nepsgrid,~]=size(epsgrid);

Results_prob=zeros(Nsim,dimbeta);

Results_ASF=zeros(Nsim,1);

Results_ASF_rob=zeros(Nsim,nepsgrid);

Results_ASF_bias=zeros(Nsim,nepsgrid);

Results_ASF_std=zeros(Nsim,nepsgrid);

Results_epsilon=zeros(Nsim,nlambdagrid-1);

% option: uncomment if you wish to obtain the "extrapolation" or "interpolation" case
xxtot=[0.5 1];
%xxtot=[-0.5 1];

% DGP
UU=rand(1000000,1);
Results_ASF_true=mean(xxtot*beta+(UU>2/3).*(2+.5*randn(1000000,1))...
    +(UU<2/3).*(-1+.3*randn(1000000,1))>0);
A=(UU>2/3).*(2+.5*randn(1000000,1))...
    +(UU<2/3).*(-1+.3*randn(1000000,1));
std1=std(A);
A=A/std1;

[f0 x0]=ksdensity(A);
% Figure 2b
figure
plot(x0,f0,'-','Color','b','Linewidth',3)
axis([-2.5 2.5 0 1.5])
xlabel('latent variable A','FontSize',15)
ylabel('density','FontSize',15)
hold off

% 2*KL(pi_0,pi_*)
pi_0=1/3*(1/(.5/std1))*normpdf((A-2/std1)/(.5/std1))+...
    2/3*(1/(.3/std1))*normpdf((A+1/std1)/(.3/std1));
pi_star=normpdf(A);
KL2=2*mean(log(pi_0./pi_star));
disp(KL2)

% Monte Carlo loop
for jsim=1:Nsim
    
    % dgp
    UU=rand(N,1);
    A=(UU>2/3).*(2+.5*randn(N,1))...
        +(UU<2/3).*(-1+.3*randn(N,1));
    A=A/sqrt(var(A));
    
    Y=(Xtot*beta+A>=0);
    
    % probit MLE
    par=fminunc(@(par)-sum(Y.*log(normcdf(Xtot*par))+(1-Y).*log(normcdf(-Xtot*par))),par0,options);
    Results_prob(jsim,:)=par';
    
    % correction
    S1=Xtot.*normpdf(Xtot*par).*(1-normcdf(Xtot*par))./(normcdf(Xtot*par).*(1-normcdf(Xtot*par)));
    S0=Xtot.*normpdf(Xtot*par).*(0-normcdf(Xtot*par))./(normcdf(Xtot*par).*(1-normcdf(Xtot*par)));
    S=[S1;S0];
    
    H=(Xtot.*(normpdf(Xtot*par)).^2./(normcdf(Xtot*par).*(1-normcdf(Xtot*par))))'*Xtot/N;
    
    W11=normcdf(min(Xtot*par,(Xtot*par)'))./normcdf(Xtot*par);
    W10=1-W11;
    W01=(Xtot*par<(Xtot*par)').*(normcdf((Xtot*par)')-normcdf((Xtot*par)))./(1-normcdf((Xtot*par)));
    W00=1-W01;
    W=(1/N)*[W11 W10;W01 W00];
    
    C1=zeros(dimbeta,N);
    C0=zeros(dimbeta,N);
    
    for jj=1:dimbeta
        
        C1(jj,:)=mean(normcdf(min((Xtot*par)',(Xtot*par))).*S1(:,jj)+...
            (Xtot*par<(Xtot*par)').*(normcdf((Xtot*par)')-normcdf((Xtot*par))).*S0(:,jj));
        
        C0(jj,:)=mean((normcdf(Xtot*par)-normcdf(min((Xtot*par)',(Xtot*par)))).*S1(:,jj)+...
            (1-normcdf(Xtot*par)-(Xtot*par<(Xtot*par)').*(normcdf((Xtot*par)')-normcdf((Xtot*par)))).*S0(:,jj));
        
    end
    
    C=(1/N)*[C1 C0];
    
    % choice of epsilon
    
    FF1=(-Xtot*par<a_kk(1:end-1)').*(normcdf(a_kk(2:end))-normcdf(a_kk(1:end-1)))'./(normcdf(Xtot*par))+...
        ((-Xtot*par>a_kk(1:end-1)').*(-Xtot*par<=a_kk(2:end)')).*(normcdf(a_kk(2:end))'-normcdf(-Xtot*par))./(normcdf(Xtot*par));
    
    FF0=(-Xtot*par>a_kk(2:end)').*(normcdf(a_kk(2:end))'-normcdf(a_kk(1:end-1))')./(1-normcdf(Xtot*par))+...
        (-Xtot*par>a_kk(1:end-1)').*(-Xtot*par<=a_kk(2:end)').*(normcdf(-Xtot*par)-normcdf(a_kk(1:end-1))')./(1-normcdf(Xtot*par));
    
    Vect=((normcdf(Xtot*par).*FF1)'*S1/N+((1-normcdf(Xtot*par)).*FF0)'*S0/N);
    
    HHtilde2=(normcdf(Xtot*par).*FF1)'*FF1/N+((1-normcdf(Xtot*par)).*FF0)'*FF0/N-Vect*inv(H)*Vect';
    
    vect_eig2=eigs(invD2*HHtilde2);
    
    Results_epsilon(jsim,:)=(2.8^2)./(N*vect_eig2(2:nlambdagrid));
    
    Results_ASF(jsim,1)=normcdf(xxtot*par);
    
    % loop on epsilon
    for jeps=1:nepsgrid
        
        epsilon=epsgrid(jeps);
        
        V1=normcdf(min(xxtot*par,Xtot*par))./normcdf(Xtot*par)-normcdf(xxtot*par);
        V0=(Xtot*par<xxtot*par).*(normcdf(xxtot*par)-normcdf(Xtot*par))./(1-normcdf(Xtot*par))-normcdf(xxtot*par);
        V=[V1;V0];
        
        AA=mean(normcdf(min(xxtot*par,Xtot*par)).*S1+...
            (Xtot*par<xxtot*par).*(normcdf(xxtot*par)-normcdf(Xtot*par)).*S0)';
        
        % least squares to compute hmmse
        aa=S*inv(H)*(xxtot'.*normpdf(xxtot*par))+(epsilon*N)*(V-S*inv(H)*AA);
        BB=(epsilon*N)*(-W+S*inv(H)*C);
        
        hMMSE=(eye(2*N)-BB)\aa;
        
        Results_ASF_rob(jsim,jeps)=Results_ASF(jsim,1)+mean(Y.*hMMSE(1:N,:)+(1-Y).*hMMSE(N+1:2*N,:));
        
        % CI
        hh=Y.*hMMSE(1:N,:)+(1-Y).*hMMSE(N+1:2*N,:);
        
        std=sqrt(var(hh)/N);
        
        Results_ASF_std(jsim,jeps)=std;
        
        hhtilde=mean((Xtot*par+a_kk'>0).*hMMSE(1:N,:))'+mean((Xtot*par+a_kk'<=0).*hMMSE(N+1:2*N,:))';
        ddelta=(xxtot*par+a_kk>0);
        bias=sqrt(epsilon)*sqrt(var(ddelta- Results_ASF(jsim,1)-hhtilde));
        
        Results_ASF_bias(jsim,jeps)=bias;
        
    end
    
    disp(jsim)
    
    
end

% Figure 4
figure
plot(epsgrid,Results_ASF_true*ones(nepsgrid,1),':','Color','k','Linewidth',2)
hold on
plot(epsgrid,mean(Results_ASF)*ones(nepsgrid,1),'Color','g','Linewidth',3)
hold on
plot(epsgrid,mean(Results_ASF_rob)','Color','b','Linewidth',3)
hold on
plot(epsgrid,quantile(Results_ASF_rob,.975)','--','Color','b')
hold on
plot(epsgrid,quantile(Results_ASF_rob,.025)','--','Color','b')

% uncomment if extra/intra - polation case
axis([0 1.2 0.1 .6])
%axis([0 1.2 -.1 .4])

vline(mean(Results_epsilon),'k-')
xlabel('epsilon','FontSize',15)
ylabel('delta','FontSize',15)
hold off

Results_ASF_CI_up=Results_ASF_rob+Results_ASF_bias+1.96*Results_ASF_std;
Results_ASF_CI_down=Results_ASF_rob-Results_ASF_bias-1.96*Results_ASF_std;

sg=[1 5 9 13 17 21]';

% Tables S1 and S2

disp('epsilon')
disp(epsgrid(sg)')

disp('w-c bias')
disp(mean(Results_ASF_bias(:,sg)))

disp('as. standard error')
disp(mean(Results_ASF_std(:,sg)))

disp('MC bias')
disp(mean(Results_ASF_rob(:,sg)-Results_ASF_true))

disp('MC standard error')
disp(sqrt(var(Results_ASF_rob(:,sg)-Results_ASF_true)))

disp('MC RMSE')
disp(sqrt(mean((Results_ASF_rob(:,sg)-Results_ASF_true).^2)))

disp('length CI')
disp(mean(Results_ASF_CI_up(:,sg)-Results_ASF_CI_down(:,sg)))

disp('coverage')
disp(mean((Results_ASF_CI_up(:,sg)>Results_ASF_true).*(Results_ASF_CI_down(:,sg)<Results_ASF_true)))


% OPTIONAL: to compute the identified set, uncomment this part
% It makes use of Gurobi with Matlab interface, and uses "linprog_gurobi.m"

% R=10000;
% S=NX+2;
%
% Slarge=50000;
%
% Xr=zeros(R,1);
%
% UXr=rand(R,1);
%
% pi_X=(1/NX:1/NX:(NX-1)/NX);
%
% for kX=1:NX
%     if kX==1
%         indX=(UXr<pi_X(1));
%     elseif kX>1 && kX<NX
%         indX=(UXr>=pi_X(kX-1)).*(UXr<pi_X(kX));
%     elseif kX==NX
%         indX=(UXr>=pi_X(NX-1));
%     end
%
%     Xr(indX==1)=(kX-1/2)/NX;
% end
%
% Xrtot=[Xr ones(R,1)];
%
%
% beta1grid=(-1:.01:4)';
% beta2grid=(-3:.01:1)';
%
%     UU=rand(Slarge,1);
%     Asl=(UU>2/3).*(2+.5*randn(Slarge,1))...
%         +(UU<2/3).*(-1+.3*randn(Slarge,1));
%     Asl=Asl/std(Asl);
%
% P=mean((Xrtot*beta+Asl'>=0),2);
%
% Results_max=-100*ones(length(beta1grid),length(beta2grid));
% Results_min=100*ones(length(beta1grid),length(beta2grid));
%
% options2=optimoptions('linprog','Display','off');
%
% for j1grid=1:length(beta1grid)
%     b1=beta1grid(j1grid);
%     for j2grid=1:length(beta2grid)
%
%         b2=beta2grid(j2grid);
%         b=[b1;b2];
%
%         Vect=-[[((1:1:NX)'-1/2)/NX ones(NX,1)]*b;xxtot*b];
%
%         As=[sort(Vect)-.001;max(Vect)+20];
%
%         As(1)=min(Vect)-20;
%
%         Q=(Xrtot*b+As'>=0);
%
%         delta=(xxtot*b+As>=0);
%
%         [pivar1,maxval,exitflag]=linprog_gurobi(-delta,[],[],[Q;ones(1,S)],[P;1],zeros(S,1),[],options2);
%
%         if exitflag~=-2
%             Results_max(j1grid,j2grid)=-maxval;
%         end
%
%         [pivar2,minval,exitflag]=linprog_gurobi(-(-delta),[],[],[Q;ones(1,S)],[P;1],zeros(S,1),[],options2);
%
%         if exitflag~=-2
%             Results_min(j1grid,j2grid)=minval;
%         end
%
%         disp([j1grid j2grid])
%     end
% end
%
% maxnumb=max(max(Results_max));
%
% minnumb=min(min(Results_min));
%
%
