% Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification"
% This code reproduces Figures 2(b)-(c) and 5, and Table 2 in the paper
% as well as Tables S3 and S4 in the appendix
% (up to some randomness due to different seeds)
% it uses "proc_lik_RE_corr.m"

clear
clc
close all

rng('shuffle')

% Sample size
N=500;

% For computation (when plotting the density of A)
Nlong=100000;

% Grid of T values
GridT=[5 10 20];
[~,nGridT]=size(GridT);

% parameters
beta=1;

% true is mixture of Gaussians

nu10_true=-1;
nu20_true=2;
nu11_true=-3;
nu21_true=0;
sig10_true=.3;
sig20_true=.5;
sig11_true=.5;
sig21_true=.3;
pi01_true=2/3;
pi02_true=1/3;
pi11_true=1/3;
pi12_true=2/3;

% number of draws for computation
Sa=1000;

% To plot density of A

UU0=rand(Nlong,1);
UU1=rand(Nlong,1);

alphalong0=(UU0<pi01_true).*(nu10_true+sig10_true*randn(Nlong,1))+...
    (UU0>=pi01_true).*(UU0<pi01_true+pi02_true).*(nu20_true+sig20_true*randn(Nlong,1));
alphalong1=(UU1<pi11_true).*(nu11_true+sig11_true*randn(Nlong,1))+...
    (UU1>=pi11_true).*(UU1<pi11_true+pi12_true).*(nu21_true+sig21_true*randn(Nlong,1));


% Plot density of A (Figure 2(b)-(c))
[f0,x0]=ksdensity(alphalong0);
[f1,x1]=ksdensity(alphalong1);
figure
plot(x0,f0,'-','Color','b','Linewidth',3)
hold on
plot(x1,f1,'--','Color','g','Linewidth',3)
axis([-4.5 4.5 0 1])
xlabel('latent variable A','FontSize',15)
ylabel('density','FontSize',15)
hold off


% True average effects
Delta00=normcdf(beta+alphalong0)-normcdf(alphalong0);
Delta01=normcdf(beta+alphalong1)-normcdf(alphalong1);

delta0=(mean(Delta00)+mean(Delta01))/2;


% Grid of epsilon
epsgrid=(.000001:.1:20.000001)';
[nepsgrid,~]=size(epsgrid);

% number of eigenvalues - for epsilon
lambdagrid=(1:1:4)';
[nlambdagrid,~]=size(lambdagrid);

% Number of Monte Carlo simulations
S=1000;

% To store the results
Results_RE=zeros(S,nGridT);
Results_EB=zeros(S,nGridT);
Results_LP=zeros(S,nGridT);
Results_betaMLE=zeros(S,nGridT);
Results_betaMMSE=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_betaBias=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_betaStd=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_MMSE=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_Bias=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_Std=zeros(S,nGridT,nepsgrid+nlambdagrid-1);
Results_epsilon=zeros(S,nGridT,nlambdagrid-1);

% options for MLE
options=optimoptions('fminunc','Display','off');

% loop on T
tic
for jt=1:nGridT
    
    T=GridT(jt);
    
    % monte carlo simulations
    for ss=1:S
        
        %%% DGP
        % Initial condition
        Y0=(rand(N,1)>1/2);
        
        %   pi0 (normal mixture)
        UU0=rand(N,1);
        UU1=rand(N,1);
        alpha0=(UU0<pi01_true).*(nu10_true+sig10_true*randn(N,1))+...
            (UU0>=pi01_true).*(UU0<pi01_true+pi02_true).*(nu20_true+sig20_true*randn(N,1));
        
        alpha1=(UU1<pi11_true).*(nu11_true+sig11_true*randn(N,1))+...
            (UU1>=pi11_true).*(UU1<pi11_true+pi12_true).*(nu21_true+sig21_true*randn(N,1));
        
        alpha=(1-Y0).*alpha0+Y0.*alpha1;
        
        % Y's (under pi_0)
        Y=zeros(N,T+1);
        Y(:,1)=Y0;
        for tt=2:T+1
            Y(:,tt)=(beta*Y(:,tt-1)+alpha+randn(N,1)>0);
        end
        
        % linear probability estimator
        
        YY=Y(:,2:T)-mean(Y(:,2:T)')'*ones(1,T-1);
        XX=Y(:,1:T-1)-mean(Y(:,1:T-1)')'*ones(1,T-1);
        
        par=pinv(XX(:))*YY(:);
        
        Results_LP(ss,jt)=par;
        
        % Random-effects estimator
        par0=[beta 0 0 0]';
        UU=rand(Sa,1);
        
        % MLE
        par=fminunc(@(a) proc_lik_RE_corr(a,N,T,Sa,Y,UU),par0,options);
        
        beta_RE=par(1);
        nu0=par(2);
        nu1=par(3);
        sig=exp(par(4));
        
        % parameter
        Results_betaMLE(ss,jt)=beta_RE;
        
        alphagrid0=nu0+sig*norminv(UU);
        alphagrid1=nu0+nu1+sig*norminv(UU);
        
        Delta0=normcdf(beta_RE+alphagrid0)-normcdf(alphagrid0);
        meanDelta0=mean(Delta0);
        
        Delta1=normcdf(beta_RE+alphagrid1)-normcdf(alphagrid1);
        meanDelta1=mean(Delta1);
        
        % Random-effects estimator - average effect
        Results_RE(ss,jt)=mean(1-Y(:,1))*mean(Delta0)+mean(Y(:,1))*mean(Delta1);
        
        % Empirical Bayes estimator
        
        gYialpha0=ones(N,Sa);
        for tt=2:T+1
            gYialpha0=gYialpha0.*((Y(:,tt)==1).*normcdf(beta_RE*Y(:,tt-1)+alphagrid0')+...
                (Y(:,tt)==0).*normcdf(-beta_RE*Y(:,tt-1)-alphagrid0'));
        end
        
        gYialpha1=ones(N,Sa);
        for tt=2:T+1
            gYialpha1=gYialpha1.*((Y(:,tt)==1).*normcdf(beta_RE*Y(:,tt-1)+alphagrid1')+...
                (Y(:,tt)==0).*normcdf(-beta_RE*Y(:,tt-1)-alphagrid1'));
        end
        
        meanDeltaYi0=gYialpha0*Delta0./(gYialpha0*ones(Sa,1));
        meanDeltaYi1=gYialpha1*Delta1./(gYialpha1*ones(Sa,1));
        
        Results_EB(ss,jt)=mean((1-Y(:,1)).*meanDeltaYi0+Y(:,1).*meanDeltaYi1);
        
        %%% MMSE estimation
        
        % simulated Y's (under pi^*)
        Ygrid0=zeros(Sa,T+1);
        Ygrid0(:,1)=zeros(Sa,1);
        for tt=2:T+1
            Ygrid0(:,tt)=(beta_RE*Ygrid0(:,tt-1)+alphagrid0+randn(Sa,1)>0);
        end
        
        Ygrid1=zeros(Sa,T+1);
        Ygrid1(:,1)=ones(Sa,1);
        for tt=2:T+1
            Ygrid1(:,tt)=(beta_RE*Ygrid1(:,tt-1)+alphagrid1+randn(Sa,1)>0);
        end
        
        % construction of vectors and matrices for minimum-MSE estimator
        % (see Appendix S3)
        
        gYalpha0=ones(Sa,Sa);
        for tt=2:T+1
            gYalpha0=gYalpha0.*((Ygrid0(:,tt)==1).*normcdf(beta_RE*Ygrid0(:,tt-1)+alphagrid0')+...
                (Ygrid0(:,tt)==0).*normcdf(-beta_RE*Ygrid0(:,tt-1)-alphagrid0'));
        end
        
        gYalpha1=ones(Sa,Sa);
        for tt=2:T+1
            gYalpha1=gYalpha1.*((Ygrid1(:,tt)==1).*normcdf(beta_RE*Ygrid1(:,tt-1)+alphagrid1')+...
                (Ygrid1(:,tt)==0).*normcdf(-beta_RE*Ygrid1(:,tt-1)-alphagrid1'));
        end
        
        G0=gYalpha0./(gYalpha0*ones(Sa,1));
        G1=gYalpha1./(gYalpha1*ones(Sa,1));
        
        iota=ones(Sa,1);
        
        % scores
        
        nabla_beta_log_gYalpha0=zeros(Sa,Sa);
        for tt=2:T+1
            nabla_beta_log_gYalpha0=nabla_beta_log_gYalpha0...
                +(Ygrid0(:,tt)==1).*Ygrid0(:,tt-1).*normpdf(beta_RE*Ygrid0(:,tt-1)+alphagrid0')./...
                normcdf(beta_RE*Ygrid0(:,tt-1)+alphagrid0')...
                -(Ygrid0(:,tt)==0).*Ygrid0(:,tt-1).*normpdf(beta_RE*Ygrid0(:,tt-1)+alphagrid0')./...
                normcdf(-beta_RE*Ygrid0(:,tt-1)-alphagrid0');
        end
        
        nabla_beta_log_gYalpha1=zeros(Sa,Sa);
        for tt=2:T+1
            nabla_beta_log_gYalpha1=nabla_beta_log_gYalpha1...
                +(Ygrid1(:,tt)==1).*Ygrid1(:,tt-1).*normpdf(beta_RE*Ygrid1(:,tt-1)+alphagrid1')./...
                normcdf(beta_RE*Ygrid1(:,tt-1)+alphagrid1')...
                -(Ygrid1(:,tt)==0).*Ygrid1(:,tt-1).*normpdf(beta_RE*Ygrid1(:,tt-1)+alphagrid1')./...
                normcdf(-beta_RE*Ygrid1(:,tt-1)-alphagrid1');
        end
        
        nabla_nu0_log_piA0=1/sig^2*(alphagrid0-nu0);
        nabla_nu0_log_piA1=1/sig^2*(alphagrid1-nu0-nu1);
        nabla_nu1_log_piA0=zeros(Sa,1);
        nabla_nu1_log_piA1=1/sig^2*(alphagrid1-nu0-nu1);
        nabla_sig_log_piA0=-1/sig+1/sig^3*(alphagrid0-nu0).^2;
        nabla_sig_log_piA1=-1/sig+1/sig^3*(alphagrid1-nu0-nu1).^2;
        
        D0=[((gYalpha0.*nabla_beta_log_gYalpha0)*ones(Sa,1))./(gYalpha0*ones(Sa,1))...
            ((gYalpha0.*nabla_nu0_log_piA0')*ones(Sa,1))./(gYalpha0*ones(Sa,1))...
            ((gYalpha0.*nabla_nu1_log_piA0')*ones(Sa,1))./(gYalpha0*ones(Sa,1))...
            ((gYalpha0.*nabla_sig_log_piA0')*ones(Sa,1))./(gYalpha0*ones(Sa,1))];
        
        D1=[((gYalpha1.*nabla_beta_log_gYalpha1)*ones(Sa,1))./(gYalpha1*ones(Sa,1))...
            ((gYalpha1.*nabla_nu0_log_piA1')*ones(Sa,1))./(gYalpha1*ones(Sa,1))...
            ((gYalpha1.*nabla_nu1_log_piA1')*ones(Sa,1))./(gYalpha1*ones(Sa,1))...
            ((gYalpha1.*nabla_sig_log_piA1')*ones(Sa,1))./(gYalpha1*ones(Sa,1))];
        
        Q0=eye(Sa)-D0*pinv(D0);
        Q1=eye(Sa)-D1*pinv(D1);
        
        Gtilde0=Q0*G0;
        Gtilde1=Q1*G1;
        
        iotatilde0=Q0*iota;
        iotatilde1=Q1*iota;
        
        nabla_beta_log_gYialpha0=zeros(N,Sa);
        for tt=2:T+1
            nabla_beta_log_gYialpha0=nabla_beta_log_gYialpha0...
                +(Y(:,tt)==1).*Y(:,tt-1).*normpdf(beta_RE*Y(:,tt-1)+alphagrid0')./normcdf(beta_RE*Y(:,tt-1)+alphagrid0')...
                -(Y(:,tt)==0).*Y(:,tt-1).*normpdf(beta_RE*Y(:,tt-1)+alphagrid0')./normcdf(-beta_RE*Y(:,tt-1)-alphagrid0');
        end
        
        nabla_beta_log_gYialpha1=zeros(N,Sa);
        for tt=2:T+1
            nabla_beta_log_gYialpha1=nabla_beta_log_gYialpha1...
                +(Y(:,tt)==1).*Y(:,tt-1).*normpdf(beta_RE*Y(:,tt-1)+alphagrid1')./normcdf(beta_RE*Y(:,tt-1)+alphagrid1')...
                -(Y(:,tt)==0).*Y(:,tt-1).*normpdf(beta_RE*Y(:,tt-1)+alphagrid1')./normcdf(-beta_RE*Y(:,tt-1)-alphagrid1');
        end
        
        DY0=[((gYialpha0.*nabla_beta_log_gYialpha0)*ones(Sa,1))./(gYialpha0*ones(Sa,1))...
            ((gYialpha0.*nabla_nu0_log_piA0')*ones(Sa,1))./(gYialpha0*ones(Sa,1))...
            ((gYialpha0.*nabla_nu1_log_piA0')*ones(Sa,1))./(gYialpha0*ones(Sa,1))...
            ((gYialpha0.*nabla_sig_log_piA0')*ones(Sa,1))./(gYialpha0*ones(Sa,1))];
        
        DY1=[((gYialpha1.*nabla_beta_log_gYialpha1)*ones(Sa,1))./(gYialpha1*ones(Sa,1))...
            ((gYialpha1.*nabla_nu0_log_piA1')*ones(Sa,1))./(gYialpha1*ones(Sa,1))...
            ((gYialpha1.*nabla_nu1_log_piA1')*ones(Sa,1))./(gYialpha1*ones(Sa,1))...
            ((gYialpha1.*nabla_sig_log_piA1')*ones(Sa,1))./(gYialpha1*ones(Sa,1))];
        
        GY0=gYialpha0./(gYialpha0*ones(Sa,1));
        GY1=gYialpha1./(gYialpha1*ones(Sa,1));
        iotaY=ones(N,1);
        
        GYtilde0=GY0-DY0*pinv(D0)*G0;
        GYtilde1=GY1-DY1*pinv(D1)*G1;
        
        iotaYtilde0=iotaY-DY0*pinv(D0)*iota;
        iotaYtilde1=iotaY-DY1*pinv(D1)*iota;
        
        invHeta=inv(mean(1-Y(:,1))*(D0'*D0)/Sa+mean(Y(:,1))*(D1'*D1)/Sa);
        
        % epsilon
        
        vect_lambda=eigs(mean(1-Y(:,1))*(Gtilde0'*Gtilde0)+mean(Y(:,1))*(Gtilde1'*Gtilde1));
        Results_epsilon(ss,jt,:)=2.8^2./(N*vect_lambda(2:nlambdagrid))';
        
        for jeps=1:nepsgrid+nlambdagrid-1
            
            ee=epsgrid(min(jeps,nepsgrid));
            
            if jeps<=nepsgrid
                epsilon=ee;
            elseif jeps>nepsgrid
                epsilon=2.8^2./(N*vect_lambda(jeps-nepsgrid+1));
            end
            
            % beta
            Meps10=Gtilde0*G0'+(epsilon*N)^(-1)*eye(Sa);
            Meps11=Gtilde1*G1'+(epsilon*N)^(-1)*eye(Sa);
            
            Veps10=(epsilon*N)^(-1)*D0*invHeta*[1;0;0;0]+(Gtilde0-iotatilde0*pinv(iota))*beta_RE*ones(Sa,1);
            Veps11=(epsilon*N)^(-1)*D1*invHeta*[1;0;0;0]+(Gtilde1-iotatilde1*pinv(iota))*beta_RE*ones(Sa,1);
            
            beta_MMSE=beta_RE+pinv(iotaY)*((1-Y(:,1)).*(DY0*invHeta*[1;0;0;0]+(epsilon*N)*...
                ((GYtilde0-iotaYtilde0*pinv(iota))*beta_RE*ones(Sa,1)-GYtilde0*G0'*(Meps10\Veps10)))+...
                Y(:,1).*(DY1*invHeta*[1;0;0;0]+(epsilon*N)*...
                ((GYtilde1-iotaYtilde1*pinv(iota))*beta_RE*ones(Sa,1)-GYtilde1*G1'*(Meps11\Veps11))));
            
            Results_betaMMSE(ss,jt,jeps)=beta_MMSE;
            
            % bias
            
            hMMSE0=(D0*invHeta*[1;0;0;0]+(epsilon*N)*...
                ((Gtilde0-iotatilde0*pinv(iota))*beta_RE*ones(Sa,1)-Gtilde0*G0'*(Meps10\Veps10)));
            hMMSE1=(D1*invHeta*[1;0;0;0]+(epsilon*N)*...
                ((Gtilde1-iotatilde1*pinv(iota))*beta_RE*ones(Sa,1)-Gtilde1*G1'*(Meps11\Veps11)));
            
            Results_betaBias(ss,jt,jeps)=sqrt(epsilon)*sqrt(1/2*var(G0'*hMMSE0)+1/2*var(G1'*hMMSE1));
            
            % standard error
            Results_betaStd(ss,jt,jeps)=sqrt(1/2*var(hMMSE0)+1/2*var(hMMSE1))/sqrt(N);
            
            % Average effect
            DerDelta0=zeros(4,1);
            DerDelta1=zeros(4,1);
            
            DerDelta0(1)=mean(normpdf(beta_RE+alphagrid0));
            DerDelta1(1)=mean(normpdf(beta_RE+alphagrid1));
            
            DerDelta0(2)=mean((normcdf(beta_RE+alphagrid0)-normcdf(alphagrid0)).*nabla_nu0_log_piA0);
            DerDelta1(2)=mean((normcdf(beta_RE+alphagrid1)-normcdf(alphagrid1)).*nabla_nu0_log_piA1);
            DerDelta0(3)=mean((normcdf(beta_RE+alphagrid0)-normcdf(alphagrid0)).*nabla_nu1_log_piA0);
            DerDelta1(3)=mean((normcdf(beta_RE+alphagrid1)-normcdf(alphagrid1)).*nabla_nu1_log_piA1);
            DerDelta0(4)=mean((normcdf(beta_RE+alphagrid0)-normcdf(alphagrid0)).*nabla_sig_log_piA0);
            DerDelta1(4)=mean((normcdf(beta_RE+alphagrid1)-normcdf(alphagrid1)).*nabla_sig_log_piA1);
            
            V2eps10=(epsilon*N)^(-1)*D0*invHeta*DerDelta0+...
                (Gtilde0-iotatilde0*pinv(iota))*Delta0;
            V2eps11=(epsilon*N)^(-1)*D1*invHeta*DerDelta1+...
                (Gtilde1-iotatilde1*pinv(iota))*Delta1;
            
            delta_MMSE=mean(1-Y(:,1))*mean(Delta0)+mean(Y(:,1))*mean(Delta1)...
                +pinv(iotaY)*((1-Y(:,1)).*(DY0*invHeta*DerDelta0+(epsilon*N)*...
                ((GYtilde0-iotaYtilde0*pinv(iota))*Delta0-GYtilde0*G0'*(Meps10\V2eps10)))...
                +Y(:,1).*(DY1*invHeta*DerDelta1+(epsilon*N)*...
                ((GYtilde1-iotaYtilde1*pinv(iota))*Delta1-GYtilde1*G1'*(Meps11\V2eps11))));
            
            Results_MMSE(ss,jt,jeps)=delta_MMSE;
            
            hMMSE20=(D0*invHeta*DerDelta0+(epsilon*N)*...
                ((Gtilde0-iotatilde0*pinv(iota))*Delta0-Gtilde0*G0'*(Meps10\V2eps10)));
            
            hMMSE21=(D1*invHeta*DerDelta1+(epsilon*N)*...
                ((Gtilde1-iotatilde1*pinv(iota))*Delta1-Gtilde1*G1'*(Meps11\V2eps11)));
            
            Results_Bias(ss,jt,jeps)=sqrt(epsilon)*sqrt(1/2*var(Delta0-G0'*hMMSE20)+1/2*var(Delta1-G1'*hMMSE21));
            
            Results_Std(ss,jt,jeps)=sqrt(1/2*var(hMMSE20)+1/2*var(hMMSE21))/sqrt(N);
            
        end
        
        disp(ss)
    end
    disp(jt)
end
toc

for tt=1:nGridT
    
    % Figure 5 (top panel)
    figure
    plot(epsgrid,delta0*ones(nepsgrid,1),':','Color','k','Linewidth',2)
    hold on
    plot(epsgrid,mean(Results_RE(:,tt))*ones(nepsgrid,1),'Color','g','Linewidth',3)
    hold on
    plot(epsgrid,mean(reshape(Results_MMSE(:,tt,1:nepsgrid),S,nepsgrid))','Color','b','Linewidth',3)
    hold on
    plot(epsgrid,quantile(reshape(Results_MMSE(:,tt,1:nepsgrid),S,nepsgrid),.975)','--','Color','b')
    hold on
    plot(epsgrid,quantile(reshape(Results_MMSE(:,tt,1:nepsgrid),S,nepsgrid),.025)','--','Color','b')
    if tt==1
        axis([0 1 .1 .35])
    elseif tt==2
        axis([0 1 .1 .35])
    elseif tt==3
        axis([0 1 .1 .35])
    end
    vline(mean(reshape(Results_epsilon(:,tt,:),S,nlambdagrid-1)),'k-')
    xlabel('epsilon','FontSize',15)
    ylabel('delta','FontSize',15)
    hold off
    
end

for tt=1:nGridT
    
    % Figure 5 (bottom panel)
    figure
    plot(epsgrid,beta*ones(nepsgrid,1),':','Color','k','Linewidth',2)
    hold on
    plot(epsgrid,mean(Results_betaMLE(:,tt))*ones(nepsgrid,1),'Color','g','Linewidth',3)
    hold on
    plot(epsgrid,mean(reshape(Results_betaMMSE(:,tt,1:nepsgrid),S,nepsgrid))','Color','b','Linewidth',3)
    hold on
    plot(epsgrid,quantile(reshape(Results_betaMMSE(:,tt,1:nepsgrid),S,nepsgrid),.975)','--','Color','b')
    hold on
    plot(epsgrid,quantile(reshape(Results_betaMMSE(:,tt,1:nepsgrid),S,nepsgrid),.025)','--','Color','b')
    if tt==1
        axis([0 1 .5 1.5])
    elseif tt==2
        axis([0 1 .5 1.5])
    elseif tt==3
        axis([0 1 .5 1.5])
    end
    vline(mean(reshape(Results_epsilon(:,tt,:),S,nlambdagrid-1)),'k-')
    xlabel('epsilon','FontSize',15)
    ylabel('beta','FontSize',15)
    hold off
    
end

Results_CI_up=Results_MMSE+Results_Bias+1.96*Results_Std;
Results_CI_down=Results_MMSE-Results_Bias-1.96*Results_Std;

Results_betaCI_up=Results_betaMMSE+Results_betaBias+1.96*Results_betaStd;
Results_betaCI_down=Results_betaMMSE-Results_betaBias-1.96*Results_betaStd;


sg=(1:2:11)';
[snepsgrid,~]=size(sg);


% Tables S3 and S4 in the appendix
for tt=1:nGridT
    disp('T=')
    disp(GridT(tt))
    
    disp('epsilon')
    disp(epsgrid(sg)')
    
    disp('w-c bias, beta')
    disp(mean(reshape(Results_betaBias(:,tt,sg),S,snepsgrid)))
    
    disp('as. standard error, beta')
    disp(mean(reshape(Results_betaStd(:,tt,sg),S,snepsgrid)))
    
    disp('MC bias, beta')
    disp(mean(reshape(Results_betaMMSE(:,tt,sg),S,snepsgrid)-beta))
    
    disp('MC standard error, beta')
    disp(std(reshape(Results_betaMMSE(:,tt,sg),S,snepsgrid)-beta))
    
    disp('coverage, beta')
    disp(mean((reshape(Results_betaCI_up(:,tt,sg),S,snepsgrid)>beta).*...
        (reshape(Results_betaCI_down(:,tt,sg),S,snepsgrid)<beta)))
    
    disp('length CI, beta')
    disp(mean(reshape(Results_betaCI_up(:,tt,sg),S,snepsgrid)-...
        reshape(Results_betaCI_down(:,tt,sg),S,snepsgrid)))
    
    disp('w-c bias, delta')
    disp(mean(reshape(Results_Bias(:,tt,sg),S,snepsgrid)))
    
    disp('as. standard error, delta')
    disp(mean(reshape(Results_Std(:,tt,sg),S,snepsgrid)))
    
    disp('MC bias, delta')
    disp(mean(reshape(Results_MMSE(:,tt,sg),S,snepsgrid)-delta0))
    
    disp('MC standard error, delta')
    disp(std(reshape(Results_MMSE(:,tt,sg),S,snepsgrid)-delta0))
    
    disp('coverage, delta')
    disp(mean((reshape(Results_CI_up(:,tt,sg),S,snepsgrid)>delta0).*...
        (reshape(Results_CI_down(:,tt,sg),S,snepsgrid)<delta0)))
    
    disp('length CI, delta')
    disp(mean(reshape(Results_CI_up(:,tt,sg),S,snepsgrid)-...
        reshape(Results_CI_down(:,tt,sg),S,snepsgrid)))
    
    
end


% Table 2
disp('Comparison')

disp('delta, bias')
disp(mean(Results_RE(:,1:nGridT)-delta0))
disp(mean(Results_EB(:,1:nGridT)-delta0))
disp(mean(Results_LP(:,1:nGridT)-delta0))
for jeps=nepsgrid+1:nepsgrid+nlambdagrid-1
    disp(mean(Results_MMSE(:,1:nGridT,jeps)-delta0))
end

disp('delta, RMSE')
disp(sqrt(mean((Results_RE(:,1:nGridT)-delta0).^2)))
disp(sqrt(mean((Results_EB(:,1:nGridT)-delta0).^2)))
disp(sqrt(mean((Results_LP(:,1:nGridT)-delta0).^2)))
for jeps=nepsgrid+1:nepsgrid+nlambdagrid-1
    disp(sqrt(mean((Results_MMSE(:,1:nGridT,jeps)-delta0).^2)))
end

disp('beta, bias')
disp(mean(Results_betaMLE(:,1:nGridT)-beta))
for jeps=nepsgrid+1:nepsgrid+nlambdagrid-1
    disp(mean(Results_betaMMSE(:,1:nGridT,jeps)-beta))
end

disp('beta, RMSE')
disp(sqrt(mean((Results_betaMLE(:,1:nGridT)-beta).^2)))
for jeps=nepsgrid+1:nepsgrid+nlambdagrid-1
    disp(sqrt(mean((Results_betaMMSE(:,1:nGridT,jeps)-beta).^2)))
end

